package com.ruoyi.scientific.controller;

import java.util.List;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import lombok.RequiredArgsConstructor;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.PageQuery;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.scientific.domain.vo.HandbookVo;
import com.ruoyi.scientific.domain.bo.HandbookBo;
import com.ruoyi.scientific.service.IHandbookService;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 项目申报指南
 *
 * @author zqjia
 * @date 2024-07-29
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/scientific/handbook1")
public class HandbookController extends BaseController {

    private final IHandbookService iHandbookService;

    /**
     * 查询项目申报指南列表
     */
    @SaCheckPermission("scientific:handbook1:list")
    @GetMapping("/list")
    public TableDataInfo<HandbookVo> list(HandbookBo bo, PageQuery pageQuery) {
        return iHandbookService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出项目申报指南列表
     */
    @SaCheckPermission("scientific:handbook1:export")
    @Log(title = "项目申报指南", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HandbookBo bo, HttpServletResponse response) {
        List<HandbookVo> list = iHandbookService.queryList(bo);
        ExcelUtil.exportExcel(list, "项目申报指南", HandbookVo.class, response);
    }

    /**
     * 获取项目申报指南详细信息
     *
     * @param handbookId 主键
     */
    @SaCheckPermission("scientific:handbook1:query")
    @GetMapping("/{handbookId}")
    public R<HandbookVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long handbookId) {
        return R.ok(iHandbookService.queryById(handbookId));
    }

    /**
     * 新增项目申报指南
     */
    @SaCheckPermission("scientific:handbook1:add")
    @Log(title = "项目申报指南", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody HandbookBo bo) {
        return toAjax(iHandbookService.insertByBo(bo));
    }

    /**
     * 修改项目申报指南
     */
    @SaCheckPermission("scientific:handbook1:edit")
    @Log(title = "项目申报指南", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody HandbookBo bo) {
        return toAjax(iHandbookService.updateByBo(bo));
    }

    /**
     * 删除项目申报指南
     *
     * @param handbookIds 主键串
     */
    @SaCheckPermission("scientific:handbook1:remove")
    @Log(title = "项目申报指南", businessType = BusinessType.DELETE)
    @DeleteMapping("/{handbookIds}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] handbookIds) {
        return toAjax(iHandbookService.deleteWithValidByIds(Arrays.asList(handbookIds), true));
    }
}
