package com.ruoyi.scientific.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.ruoyi.scientific.domain.bo.HandbookBo;
import com.ruoyi.scientific.domain.vo.HandbookVo;
import com.ruoyi.scientific.domain.Handbook;
import com.ruoyi.scientific.mapper.HandbookMapper;
import com.ruoyi.scientific.service.IHandbookService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 项目申报指南Service业务层处理
 *
 * @author zqjia
 * @date 2024-07-29
 */
@RequiredArgsConstructor
@Service
public class HandbookServiceImpl implements IHandbookService {

    private final HandbookMapper baseMapper;

    /**
     * 查询项目申报指南
     */
    @Override
    public HandbookVo queryById(Long handbookId){
        return baseMapper.selectVoById(handbookId);
    }

    /**
     * 查询项目申报指南列表
     */
    @Override
    public TableDataInfo<HandbookVo> queryPageList(HandbookBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<Handbook> lqw = buildQueryWrapper(bo);
        Page<HandbookVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询项目申报指南列表
     */
    @Override
    public List<HandbookVo> queryList(HandbookBo bo) {
        LambdaQueryWrapper<Handbook> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<Handbook> buildQueryWrapper(HandbookBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<Handbook> lqw = Wrappers.lambdaQuery();
        lqw.eq(bo.getHandbookId() != null, Handbook::getHandbookId, bo.getHandbookId());
        lqw.like(StringUtils.isNotBlank(bo.getHandbookName()), Handbook::getHandbookName, bo.getHandbookName());
        lqw.eq(StringUtils.isNotBlank(bo.getHandbookCategory()), Handbook::getHandbookCategory, bo.getHandbookCategory());
        lqw.eq(StringUtils.isNotBlank(bo.getHandbookLevel()), Handbook::getHandbookLevel, bo.getHandbookLevel());
        lqw.eq(bo.getHandbookDate() != null, Handbook::getHandbookDate, bo.getHandbookDate());
        lqw.eq(StringUtils.isNotBlank(bo.getHandbookNote()), Handbook::getHandbookNote, bo.getHandbookNote());
        lqw.eq(StringUtils.isNotBlank(bo.getHandbookFile()), Handbook::getHandbookFile, bo.getHandbookFile());
        lqw.eq(StringUtils.isNotBlank(bo.getHandbookProcId()), Handbook::getHandbookProcId, bo.getHandbookProcId());
        lqw.eq(StringUtils.isNotBlank(bo.getHandbookStatus()), Handbook::getHandbookStatus, bo.getHandbookStatus());
        return lqw;
    }

    /**
     * 新增项目申报指南
     */
    @Override
    public Boolean insertByBo(HandbookBo bo) {
        Handbook add = BeanUtil.toBean(bo, Handbook.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setHandbookId(add.getHandbookId());
        }
        return flag;
    }

    /**
     * 修改项目申报指南
     */
    @Override
    public Boolean updateByBo(HandbookBo bo) {
        Handbook update = BeanUtil.toBean(bo, Handbook.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(Handbook entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除项目申报指南
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
